@@ -1,9 +1,9 @@
 ;;; -*- mode: Emacs-Lisp; coding: euc-japan -*-
 
-;; Author:  TSUCHIYA Masatoshi <tsuchiya@pine.kuee.kyoto-u.ac.jp>,
+;; Author:  TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;          Hideyuki SHIRAI <shirai@quickhack.net>
 ;; Created: 2000/04/25
-;; Revised: $Date$
+;; Revised: 2007-09-30 06:51:10
 
 
 ;;; Commentary:
@@ -54,7 +54,7 @@
 
 ;;; Customize Variables:
 (defcustom mhc-cvs-rsh
-  (and (getenv "CVS_RSH") "ssh")
+  (or (getenv "CVS_RSH") "ssh")
   "*The name of the remote shell command to use when starting a CVS server."
   :group 'mhc
   :type '(choice
@@ -108,18 +108,24 @@ from 'month before last' to 'this month next year'."
 
 (defsubst mhc-cvs/backend (options)
   "ꤵ줿ץդä CVS ¹Ԥؿ"
-  (let ((buffer (mhc-get-buffer-create mhc-cvs/tmp-buffer-name))
-	(current-buffer (current-buffer)))
-    (unwind-protect
-	(progn
-	  (set-buffer buffer)
-	  (delete-region (point-min) (point-max))
-	  (let ((default-directory (file-name-as-directory mhc-cvs/default-directory))
-		(process-environment process-environment))
-	    (setenv "CVS_RSH" mhc-cvs-rsh)
-	    (apply #'call-process "cvs" nil t nil
-		   (append mhc-cvs-global-options options))))
-      (set-buffer current-buffer))))
+  (let* ((buffer (mhc-get-buffer-create mhc-cvs/tmp-buffer-name))
+	 (current-buffer (current-buffer))
+	 (ret
+	  (unwind-protect
+	      (progn
+		(set-buffer buffer)
+		(delete-region (point-min) (point-max))
+		(let ((default-directory (file-name-as-directory
+					  mhc-cvs/default-directory))
+		      (process-environment (copy-sequence process-environment)))
+		  (setenv "CVS_RSH" mhc-cvs-rsh)
+		  (apply #'call-process "cvs" nil t nil
+			 (append mhc-cvs-global-options options))))
+	    (set-buffer current-buffer))))
+    (if (numberp ret)
+	ret
+      (message "error: mhc-cvs/backend: %s" ret)
+      -1)))
 
 (defun mhc-cvs/open (&optional offline)
   "ͥåȥξ֤˰¸볫Ͻؿ"
@@ -260,7 +266,7 @@ from 'month before last' to 'this month next year'."
   "եؿ"
   (let ((added (mhc-cvs/get-added-flag-file-name filename))
 	(removed (mhc-cvs/get-removed-file-name filename))
-    	(new-path (expand-file-name
+	(new-path (expand-file-name
 		   "trash"
 		   (mhc-summary-folder-to-path mhc-base-folder))))
     (or (file-directory-p new-path)
@@ -278,13 +284,19 @@ from 'month before last' to 'this month next year'."
       (if (file-exists-p removed) (delete-file removed))
       (if (file-exists-p filename)
 	  (rename-file filename (mhc-misc-get-new-path new-path)))
-      (and (= 0 (mhc-cvs/backend (list "remove" (mhc-cvs/shrink-file-name filename))))
+      (and (= 0 (mhc-cvs/backend (list "remove"
+				       (mhc-cvs/shrink-file-name filename))))
 	   (mhc-cvs/modify filename)))))
 
 (defun mhc-cvs/modify (filename &optional offline)
   "եѹؿ"
-  (or offline (= 0 (mhc-cvs/backend
-		    (list "commit" "-m" "" (mhc-cvs/shrink-file-name filename))))))
+  (or offline
+      (prog1 (= 0 (mhc-cvs/backend
+		   (list "commit" "-m" ""
+			 (mhc-cvs/shrink-file-name filename))))
+	(let ((modes (file-modes filename)))
+	  (when modes
+	    (set-file-modes filename (logior ?\200 modes)))))))
 
 
 ;;; CVS Backend Function
@@ -451,20 +463,19 @@ from 'month before last' to 'this month next year'."
       (when mhcp
 	(setq loop t)
 	(while loop
-	  (message
-	   (format "[file: %s] ? A)dd CVS repository, R)emove immediately, M)ove to trash"
-		   file))
+	  (message "[file: %s] ? A)dd CVS repository, R)emove immediately, M)ove to trash"
+		   file)
 	  (condition-case nil
 	      (setq char (read-char))
 	    (error (setq char ?Z)))	;; dummy set
 	  (cond
 	   ((memq char '(?a ?A))
 	    (setq loop nil)
-	    (message (format "[file: %s]  Add CVS repository..." file))
+	    (message "[file: %s]  Add CVS repository..." file)
 	    (mhc-record/append-log record 'add)
 	    (and (= 0 (mhc-cvs/backend (list "add" file)))
 		 (mhc-cvs/modify expf))
-	    (message (format "[file: %s]  Add CVS repository... done." file)))
+	    (message "[file: %s]  Add CVS repository...done" file))
 	   ((memq char '(?r ?R))
 	    (setq loop nil)
 	    (message "")
