@@ -1,10 +1,10 @@
 ;;; mhc-date.el -- Digit style Date Calculation Lib.
 
 ;; Author:  Yoshinari Nomura <nom@quickhack.net>,
-;;          TSUCHIYA Masatoshi <tsuchiya@pine.kuee.kyoto-u.ac.jp>
+;;          TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 ;;
 ;; Created: 2000/06/14
-;; Revised: $Date: 2000/12/14 10:39:55 $
+;; Revised: 2004-05-06 16:35:13
 
 ;;;
 ;;; Commentary:
@@ -190,6 +190,9 @@
        (/ xx  400)
        -719163)))
 
+(defsubst mhc-date/iso-week-days (yday wday)
+  (- yday -3 (% (- yday wday -382) 7)))
+
 (defmacro mhc-date/substring-to-int (str pos)
   `(string-to-int
     (substring ,str (match-beginning ,pos) (match-end ,pos))))
@@ -400,6 +403,18 @@
 (defmacro mhc-date-oo (date)
   `(/ (1- (mhc-date-dd ,date)) 7))
 
+(defsubst mhc-date-cw (date)
+  (mhc-date-let date
+    (let* ((yday (mhc-date/day-number yy mm dd))
+	   (days (mhc-date/iso-week-days yday ww))
+	   (d))
+      (if (< days 0)
+	  (setq days (mhc-date/iso-week-days 
+		      (+ yday 365 (if (mhc-date/leap-year-p (1- yy)) 1 0)) ww))
+	(setq d (mhc-date/iso-week-days
+		 (- yday 365 (if (mhc-date/leap-year-p yy) 1 0)) ww))
+	(if (<= 0 d) (setq days d)))
+      (1+ (/ days 7)))))
 
 ;;
 ;; compare.
@@ -496,6 +511,13 @@
 (defalias 'mhc-date-p 'integerp)
 
 
+;;
+;; miscellaneous.
+;;
+
+(defmacro mhc-end-day-of-week ()
+  `(nth mhc-start-day-of-week '(6 0 1 2 3 4 5)))
+
 ;;
 ;; to string.
 ;;
