@@ -3,7 +3,7 @@
 ;; Author:  Yoshinari Nomura <nom@quickhack.net>
 ;;
 ;; Created: 1999/04/13
-;; Revised: $Date: 2001/06/25 05:40:00 $
+;; Revised: 2007-12-05 04:59:35
 ;;
 
 ;;;
@@ -48,7 +48,8 @@
 ;;    $B$N$h$&$J>r7o$H2CE@(B/$B8:E@$rI=$9(B mhc-guess-keyword-score-alist $B$K4p(B
 ;;    $B$E$$$F:NE@$r$9$k!#(B
 ;;
-;; 3. $BF@E@=g$K!"(Bsort $B$7$FJV$9(B
+;; 3. $BF@E@=g(B ($BF@E@$,F1$8>l9g$O!$F|IU$d;~4V$rI=$o$9J8;zNs$,D9$$=g(B)
+;;    $B$K(B sort $B$7$FJV$9(B
 
 ;;;
 ;;; Code:
@@ -57,6 +58,23 @@
 (require 'mhc-date)
 (provide 'mhc-guess)
 
+;;; Customize variables:
+
+(defcustom mhc-guess-ignore-english-date nil
+  "*Ignore English dates."
+  :group 'mhc
+  :type '(choice (const :tag "Ignore" t)
+		 (const :tag "Don't Ignore" nil)))
+
+(defcustom mhc-guess-english-date-format '(usa)
+  "*English date formats.
+You can specify following symbols as a list.
+    usa: Suppose the USA style date formats. (e.g. Feb 25, 2004)
+    british: Suppose British style date formats. (e.g. 25 Feb, 2004)"
+  :group 'mhc
+  :type '(repeat (choice (const :tag "USA" usa)
+			 (const :tag "British" british))))
+
 ;;
 ;; regexp for get date strings.
 ;;
@@ -68,14 +86,16 @@
 	      "\\([$B#0(B-$B#9(B0-9][$B#0(B-$B#9(B0-9]\\)")
      mhc-guess/make-date-from-yyyymmdd 1 2 3)
 
-    (,(concat "\\([$BMh:##0(B-$B#9(B0-9]+\\)[\n$B!!(B ]*$B7n(B[\n $B!!(B]*$B$N(B?[\n $B!!(B]*"
+    (,(concat "\\([$B#0(B-$B#9(B0-9]+$BG/(B\\)?"
+	      "\\([$BMh:##0(B-$B#9(B0-9]+\\)[\n$B!!(B ]*$B7n(B[\n $B!!(B]*$B$N(B?[\n $B!!(B]*"
 	      "\\([$B#0(B-$B#9(B0-9]+\\)$BF|(B?"
 	      "\\([$B!J!K(B()$B7n2P?eLZ6bEZF|MK(B\n $B!!(B ]*"
-	           "\\([$B!A!](B-$B!$(B,$B!"(B]\\|$B$+$i(B\\|$B$h$j(B\\)[\n $B!!(B]*"
-		   "\\(\\([$BMh:##0(B-$B#9(B0-9]+\\)[\n$B!!(B ]*$B7n(B\\)?[\n $B!!(B]*$B$N(B?[\n $B!!(B]*"
-		   "\\([$B#0(B-$B#9(B0-9]+\\)$BF|(B?\\($B4V(B\\)?"
-	       "\\)?")
-     mhc-guess/make-date-from-mmdd 1 2 6 7 8)
+	      "\\([$B!A!](B-$B!$(B,$B!"(B]\\|$B$+$i(B\\|$B$h$j(B\\)[\n $B!!(B]*"
+	      "\\([$B#0(B-$B#9(B0-9]+$BG/(B\\)?"
+	      "\\(\\([$BMh:##0(B-$B#9(B0-9]+\\)[\n$B!!(B ]*$B7n(B\\)?[\n $B!!(B]*$B$N(B?[\n $B!!(B]*"
+	      "\\([$B#0(B-$B#9(B0-9]+\\)$BF|(B?\\($B4V(B\\)?"
+	      "\\)?")
+     mhc-guess/make-date-from-mmdd 2 3 8 9 10)
 
     (,(concat "\\([$B#0(B-$B#9(B0-9]+[$B!!(B ]*[$B!?(B/][$B!!(B ]*\\)?"
 	      "\\([$B#0(B-$B#9(B0-9]+\\)[$B!!(B ]*[$B!?(B/][$B!!(B ]*\\([$B#0(B-$B#9(B0-9]+\\)"
@@ -86,23 +106,64 @@
  		   "\\([$B#0(B-$B#9(B0-9]+\\)$BF|(B?\\($B4V(B\\)?"
 	      "\\)?")
      mhc-guess/make-date-from-mmdd 2 3 8 9 10)
-    
+
+    ;; USA style date format
+    (,(concat "\\(Jan\\(uary\\)?\\|Feb\\(ruary\\)?\\|Mar\\(ch\\)?\\|"
+	      "Apr\\(il\\)?\\|May\\|June?\\|July?\\|Aug\\(ust\\)?\\|"
+	      "Sep\\(tember\\)?\\|Oct\\(ober\\)?\\|"
+	      "Nov\\(ember\\)?\\|Dec\\(ember\\)?\\)"
+	      "\.?,? +"
+	      "\\([0-9][0-9]?\\)\\(st\\|nd\\rd\\|th\\)?,?[ \n]+" ;; day
+	      "\\(\\('\\|[1-9][0-9]\\)?[0-9][0-9]\\)?") ;; year
+     mhc-guess/make-date-from-usa-style-date 1 11 13)
+
+    ;; British style date format
+    (,(concat "\\([0-9][0-9]?\\)\\(st\\|nd\\rd\\|th\\)?,? " ;; day
+	      "\\(Jan\\(uary\\)?\\|Feb\\(ruary\\)?\\|Mar\\(ch\\)?\\|"
+	      "Apr\\(il\\)?\\|May\\|June?\\|July?\\|Aug\\(ust\\)?\\|"
+	      "Sep\\(tember\\)?\\|Oct\\(ober\\)?\\|"
+	      "Nov\\(ember\\)?\\|Dec\\(ember\\)?\\)"
+	      "\.?,?[ \n]+"
+	      "\\(\\('\\|[1-9][0-9]\\)?[0-9][0-9]\\)?") ;; year
+     mhc-guess/make-date-from-british-style-date 1 3 13)
+
     throw
 
     (,(concat "\\($B:#EY(B\\|[$B:#Mh<!(B]$B=5(B\\|$B:FMh=5(B\\)[\n $B!!(B]*$B$N(B?[\n $B!!(B]*"
 	     "\\([$B7n2P?eLZ6bEZF|(B]\\)$BMK(B")
      mhc-guess/make-date-from-relative-week 1 2)
 
+    (,(concat "\\([Tt]his\\|[Nn]ext\\)[\n ]+"
+              "\\(Monday\\|Tuesday\\|Wednesday\\|Thursday\\|Friday\\|"
+	      "Saturday\\|Sunday\\)")
+     mhc-guess/make-date-from-english-relative-week 2 1 nil)
+
+    (,(concat "\\(Monday\\|Tuesday\\|Wednesday\\|Thursday\\|Friday\\|"
+	      "Saturday\\|Sunday\\)[\n ]+"
+	      "\\([Tt]his\\|[Nn]ext\\)[ \n]+\\([Ww]eek\\)")
+     mhc-guess/make-date-from-english-relative-week 1 2 3)
+
     throw
 
     ("\\([$B#0(B-$B#9(B0-9]+\\)[\n $B!!(B]*$BF|(B"
      mhc-guess/make-date-from-mmdd nil 1)
 
+    ("\\([$B#0(B-$B#9(B0-9]+\\)[ $B!!(B]*[($B!J(B][$B7n2P?eLZ6bEZF|(B]"
+     mhc-guess/make-date-from-mmdd nil 1)
+
     ("[^\($B!J(B]\\([$B7n2P?eLZ6bEZF|(B]\\)\n?$BMK(B"
      mhc-guess/make-date-from-relative-week nil 1)
 
+    (,(concat "\\(Monday\\|Tuesday\\|Wednesday\\|Thursday\\|Friday\\|"
+	      "Saturday\\|Sunday\\)")
+     mhc-guess/make-date-from-english-relative-week 1 nil nil)
+
     ("\\($BK\F|(B\\|$B:#F|(B\\|$B$"$9(B\\|$B$"$7$?(B\\|$B$"$5$C$F(B\\|$BL@F|(B\\|$BL@8eF|(B\\)"
      mhc-guess/make-date-from-relative-day 1)
+
+    (,(concat "\\([Tt]oday\\|[Tt]omorrow\\|"
+	      "[Tt]he[ \n]+[Dd]ay[ \n]+[Aa]fter[ \n]+[Tt]omorrow\\)")
+     mhc-guess/make-date-from-english-relative-day 1)
     ))
 
 (defvar mhc-guess-time-regexp-list
@@ -113,10 +174,24 @@
      mhc-guess/make-time-from-hhmm 1 2 5 7 6)
     (,(concat "\\([$B#0(B-$B#9(B0-9]+\\)[$B!'(B:]\\([$B#0(B-$B#9(B0-9]+\\)"
 	      "\\([\n $B!!(B]*\\([$B!A!](B-]\\|$B$+$i(B\\|$B$h$j(B\\)[\n $B!!8aA08e(B]*"
-	      "\\([$B#0(B-$B#9(B0-9]+\\) *[::$B;~(B]\\($B4V(B\\)? *\\([$B#0(B-$B#9(B0-9]+\\|$BH>(B\\)?\\)?")
+	      "\\([$B#0(B-$B#9(B0-9]+\\) *[$B!'(B:$B;~(B]\\($B4V(B\\)? *\\([$B#0(B-$B#9(B0-9]+\\|$BH>(B\\)?\\)?")
      mhc-guess/make-time-from-hhmm 1 2 5 7 6)
     ))
 
+(defvar mhc-guess-location-list '()
+  "*List of the regexps of the location, like this
+  '(\"$BBh(B?[0-9$B#0(B-$B#9!;(B-$B6e(B]+$B1~@\<<(B?\"
+    \"$BBh(B?[0-9$B#0(B-$B#9!;(B-$B6e(B]+$B2q5D<<(B[0-9$B#0(B-$B#9!;(B-$B6e(B]?\"))")
+
+(defvar mhc-guess-location-regexp-list
+  `(
+    (,(concat "$B>l(B[ $B!!(B]*$B=j(B[ $B!!(B]*[$B!'(B:]*[\n $B!!(B]*\\([^\n $B!!(B]+\\)")
+     mhc-guess/make-location-from-string 1)
+    (,(concat "$B1w(B[ $B!!(B]*\\([^\n $B!!(B]+\\)")
+     mhc-guess/make-location-from-string 1)
+    (,(concat "[@$B!w(B][$B!!(B ]*\\([^\n$B!!(B .]+\\)[$B!!(B \n]")
+     mhc-guess/make-location-from-string 1)))
+
 ;; keyword to score-alist:
 ;;    each element consists of (regexp relative-boundary sameline? score)
 (defvar mhc-guess-keyword-score-alist
@@ -136,6 +211,8 @@
     ("^\\([ a-zA-Z]*>\\)+ *"                        -200 t   -15)
     ))
 
+(defvar mhc-guess/location-regexp-list nil)
+
 ;;
 ;; manipulate guess-candidate structure.
 ;;
@@ -177,6 +254,21 @@
 (defun mhc-guess-time (&optional hint1)
   (mhc-guess/guess mhc-guess-time-regexp-list hint1))
 
+(defun mhc-guess-location-setup ()
+  (if mhc-guess-location-list
+      (let ((list mhc-guess-location-list)
+	    regex)
+	(while list
+	  (setq regex (concat regex "\\(" (car list) "\\)"))
+	  (setq list (cdr list))
+	  (when list (setq regex (concat regex "\\|"))))
+ 	(setq mhc-guess/location-regexp-list
+ 	      (cons `(,regex mhc-guess/make-location-from-string 0)
+ 		    mhc-guess-location-regexp-list)))
+    (setq mhc-guess/location-regexp-list mhc-guess-location-regexp-list)))
+  
+(defun mhc-guess-location (&optional hint1)
+  (mhc-guess/guess mhc-guess/location-regexp-list hint1))
 
 (defun mhc-guess/guess (control-regexp-lst &optional hint1 now)
   (let ((score-list
@@ -184,9 +276,13 @@
 			  mhc-guess-keyword-score-alist
 			  hint1
 			  now)))
-    (sort score-list (function (lambda (a b) (< (mhc-guess-get-score b)
-						(mhc-guess-get-score a)))))))
-
+    (sort score-list 
+	  (function (lambda (a b) 
+		      (if (= (mhc-guess-get-score a) (mhc-guess-get-score b))
+			  (< (- (mhc-guess-get-end b) (mhc-guess-get-begin b))
+			     (- (mhc-guess-get-end a) (mhc-guess-get-begin a)))
+			(< (mhc-guess-get-score b)
+			   (mhc-guess-get-score a))))))))
 ;;
 ;; gather date/time.
 ;;
@@ -297,6 +393,35 @@
     (store-match-data data)
     date))
 
+(defun mhc-guess/make-date-from-usa-style-date (now month-str dd-str yy-str)
+  (if (and (null mhc-guess-ignore-english-date)
+	   (memq 'usa mhc-guess-english-date-format))
+      (mhc-guess/make-date-from-english-date now month-str dd-str yy-str)))
+
+(defun mhc-guess/make-date-from-british-style-date (now dd-str month-str yy-str)
+  (if (and (null mhc-guess-ignore-english-date)
+	   (memq 'british mhc-guess-english-date-format))
+      (mhc-guess/make-date-from-english-date now month-str dd-str yy-str)))
+
+(defun mhc-guess/make-date-from-english-date (now month-str dd-str yy-str)
+  (let* ((month-alist
+	  '(("Jan" . "1") ("Feb" . "2") ("Mar" . "3") ("Apr" . "4")
+	    ("May" . "5") ("Jun" . "6") ("Jul" . "7") ("Aug" . "8")
+	    ("Sep" . "9") ("Oct" . "10") ("Nov" . "11") ("Dec" . "12")))
+	 (mm-str (cdr (assoc (capitalize (substring month-str 0 3))
+			     month-alist)))
+	 (yy-length (length yy-str)))
+    (cond ((= yy-length 4)		; "yyyy"
+	   (mhc-guess/make-date-from-yyyymmdd now yy-str mm-str dd-str))
+	  ((or (= yy-length 3) (= yy-length 2)) ; "'yy" or "yy"
+	   (mhc-guess/make-date-from-yyyymmdd
+	    now
+	    (concat (substring (format-time-string "%Y") 0 2)
+		    (substring yy-str -2))
+	    mm-str dd-str))
+	  (t
+	   (mhc-guess/make-date-from-mmdd now mm-str dd-str)))))
+
 (defun mhc-guess/make-date-from-relative-day (now rel-word)
   (cond 
    ((null rel-word)
@@ -311,6 +436,19 @@
  	(string= rel-word "$BL@8eF|(B"))
     (cons (mhc-date+ now 2) nil))))
 
+(defun mhc-guess/make-date-from-english-relative-day (now rel-word)
+  (unless mhc-guess-ignore-english-date
+    (let ((rel (downcase rel-word)))
+      (cond 
+       ((null rel)
+	nil)
+       ((string= rel "today")
+	(cons now nil))
+       ((string= rel "tomorrow")
+	(cons (mhc-date++ now) nil))
+       (t ;; the day after tommorow.
+	(cons (mhc-date+ now 2) nil))))))
+
 (defun mhc-guess/make-date-from-relative-week (now rel-word week)
   (let ((data (match-data))
  	(ww (string-match week "$BF|7n2P?eLZ6bEZ(B"))
@@ -333,6 +471,30 @@
     (cons (mhc-date+ date off) nil)
     ))
 
+(defun mhc-guess/make-date-from-english-relative-week (now dow rel-word week)
+  (unless mhc-guess-ignore-english-date
+    (let ((dow-alist '(("Monday" . "$B7n(B") ("Tuesday" . "$B2P(B")
+		       ("Wednesday" . "$B?e(B") ("Thursday" . "$BLZ(B")
+		       ("Friday" . "$B6b(B") ("Saturday" . "$BEZ(B")
+		       ("Sunday" . "$BF|(B")))
+	  (rel (when (stringp rel-word)
+		 (downcase rel-word))))
+      (mhc-guess/make-date-from-relative-week
+       now
+       (if (null rel)
+	   nil
+	 (cond ((and (string= rel "this") (null week))
+		"$B:#EY(B")
+	       ((and (string= rel "this") week)
+		"$B:#=5(B")
+	       ((and (string= rel "next") (null week))
+		"$B:#EY(B")
+	       ((and (string= rel "next") week)
+		"$BMh=5(B")
+	       (t
+		nil)))
+       (cdr (assoc-ignore-case dow dow-alist))))))
+
 ;;
 ;; make time from string.
 ;;
@@ -366,6 +528,13 @@
 	   (t                        (mhc-guess/string-to-int mm-str))))
       (mhc-time-new xHH xMM t))))
 
+;;
+;; make location from string
+;;
+
+(defun mhc-guess/make-location-from-string (now str)
+  (cons str nil))
+
 ;;
 ;; scoring
 ;;
@@ -411,11 +580,11 @@
     (save-excursion
       (goto-char ptr)
       (if sameline
-	  (setq pmax (min pmax (save-excursion (end-of-line)       (point)))
-		pmin (max pmin (save-excursion (beginning-of-line) (point)))))
+          (setq pmax (min pmax (save-excursion (end-of-line)       (point)))
+                pmin (max pmin (save-excursion (beginning-of-line) (point)))))
       (if (< 0 rel-boundary)
-	  (search-forward-regexp regexp pmax t)
-	(search-backward-regexp regexp pmin t)))))
+          (and (< (point) pmax) (search-forward-regexp regexp pmax t))
+        (and (< pmin (point)) (search-backward-regexp regexp pmin t))))))
 
 ;;
 ;; string-to-int with code conversion.
