@@ -1,9 +1,9 @@
 ;;; -*- mode: Emacs-Lisp; coding: euc-japan -*-
 
 ;; Author:  Yoshinari Nomura <nom@quickhack.net>,
-;;          TSUCHIYA Masatoshi <tsuchiya@pine.kuee.kyoto-u.ac.jp>
+;;          TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 ;; Created: 2000/05/01
-;; Revised: $Date$
+;; Revised: 2006-04-25 05:26:12
 
 
 ;;; Commentary:
@@ -73,10 +73,22 @@
 
 ;;; Global Variables:
 
-(defcustom mhc-use-week-separator 6
-  "*If number 0 .. 6, insert separator in summary buffer."
+(defcustom mhc-summary-language 'english
+  "*Language of the summary."
   :group 'mhc
-  :type 'integer)
+  :type '(choice (const :tag "English" english)
+		 (const :tag "Japanese" japanese)))
+
+(defcustom mhc-summary-use-cw nil
+  "*If non-nil, insert `Calendar week number' instead of `Monday'."
+  :group 'mhc
+  :type '(choice (const :tag "Use" t)
+		 (const :tag "No" nil)))
+
+(defcustom mhc-use-week-separator t
+  "*If non-nil insert separator in summary buffer."
+  :group 'mhc
+  :type 'boolean)
 
 (defcustom mhc-summary-separator ?-
   "*Character of the separator as 'mhc-use-week-separator'."
@@ -84,9 +96,11 @@
   :type 'character)
 
 (defcustom mhc-use-month-separator t
-  "*If non-nil, insert separator in summary buffer for wide scope."
+  "*Insert separator in summary buffer for wide scope."
   :group 'mhc
-  :type 'boolean)
+  :type '(choice (const :tag "Insert (full width)" t)
+		 (integer :tag "Insert (number of width)")
+		 (const :tag "Not use" nil)))
 
 (defcustom mhc-summary-month-separator ?=
   "*Character of the separator as 'mhc-use-month-separator'."
@@ -124,7 +138,15 @@
   :group 'mhc
   :type 'boolean)
 
-(defcustom mhc-summary-line-format "%M%/%D %W %b%e %c%i%s %p%l"
+(defcustom mhc-insert-overdue-todo nil
+  "*Display overdue TODO on TODAY."
+  :group 'mhc
+  :type 'boolean)
+
+(defcustom mhc-summary-line-format
+  (if (eq mhc-summary-language 'japanese)
+      "%M%%D%%(%%) %b%e %c%i%s %p%l"
+    "%M%/%D%S%W %b%e %c%i%s %p%l")
   "*A format string for summary line of MHC.
 It may include any of the following format specifications
 which are replaced by the given information:
@@ -142,6 +164,14 @@ which are replaced by the given information:
 %l The location of the schedule.
 
 %/ A slash character if first line of the day.
+%( A left parenthesis character if first line of the day.
+%) A right parenthesis character if first line of the day.
+%S A space with face.
+
+%ǯ The 'ǯ' of the line if first line of the day.
+% The '' of the line if first line of the day.
+% The '' of the line if first line of the day.
+% The japaneses weekday name of the line if first line of the day.
 "
   :group 'mhc
   :type 'string)
@@ -151,6 +181,27 @@ which are replaced by the given information:
 It may include any of the following format specifications
 which are replaced by the given information:
 
+%i The icon for the schedule.
+%c The checkbox of the TODO.
+%s The subject of the schedule.
+%l The location of the schedule.
+%p The priority of the schedule.
+%d The deadline of the schedule.
+\(`mhc-todo-string-remaining-day' or `mhc-todo-string-deadline-day' is used\)
+"
+  :group 'mhc
+  :type 'string)
+
+(defcustom mhc-overdue-todo-line-format
+  (concat (if (eq mhc-summary-language 'japanese)
+	      "             "
+	    "          ")
+	  "%T  %p %c%i%s %l%d")
+  "*A format string for summary overdue todo line of MHC.
+It may include any of the following format specifications
+which are replaced by the given information:
+
+%T The indicator for TODO.
 %i The icon for the schedule.
 %c The checkbox of the TODO.
 %s The subject of the schedule.
@@ -171,24 +222,30 @@ which are replaced by the given information:
 ;;		(const :tag "Below of vertical calender" 'below))
   )
 
-(defcustom mhc-todo-string-remaining-day "( %d )"
+(defcustom mhc-todo-string-remaining-day
+  (if (eq mhc-summary-language 'japanese) "( %d )" "(%d days to go)")
   "*String format which is displayed in TODO entry.
 '%d' is replaced with remaining days."
   :group 'mhc
   :type 'string)
 
-(defcustom mhc-todo-string-deadline-day "()"
+(defcustom mhc-todo-string-deadline-day
+    (if (eq mhc-summary-language 'japanese) "()" "(due this date)")
   "*String which indicates deadline day in TODO."
   :group 'mhc
   :type 'string)
 
-(defcustom mhc-todo-string-excess-day "(%d Ķ)"
+(defcustom mhc-todo-string-excess-day
+    (if (eq mhc-summary-language 'japanese) "(%d Ķ)" "(%d days overdue)")
   "*String format which is displayed in TODO entry.
 '%d' is replaced with excess days."
   :group 'mhc
   :type 'string)
 
-(defcustom mhc-todo-string-heading "TODO(s) at %04d/%02d/%02d"
+(defcustom mhc-todo-string-heading
+      (if (eq mhc-summary-language 'japanese) 
+	  "TODO(s) at %04dǯ%02d%02d"
+	"TODO(s) at %04d/%02d/%02d")
   "*String which is displayed as heading of TODO.
 First %d is replaced with year, second one is replaced with month,
 third one is replaced with day of month."
@@ -201,12 +258,14 @@ third one is replaced with day of month."
   :type 'integer)
 
 
-(defcustom mhc-todo-string-done ""
+(defcustom mhc-todo-string-done
+  (if (eq mhc-summary-language 'japanese) "" "[X]")
   "*String which indicates done TODO."
   :group 'mhc
   :type 'string)
 
-(defcustom mhc-todo-string-not-done ""
+(defcustom mhc-todo-string-not-done
+  (if (eq mhc-summary-language 'japanese) "" "[ ]")
   "*String which indicates not-done TODO."
   :group 'mhc
   :type 'string)
@@ -216,6 +275,24 @@ third one is replaced with day of month."
   :group 'mhc
   :type 'boolean)
 
+(defcustom mhc-memo-line-format "   %p %i%s %l"
+  "*A format string for summary memo line of MHC.
+It may include any of the following format specifications
+which are replaced by the given information:
+
+%i The icon for the schedule.
+%s The subject of the schedule.
+%l The location of the schedule.
+%p The priority of the schedule.
+"
+  :group 'mhc
+  :type 'string)
+
+(defcustom mhc-memo-string-heading "MEMO(s)"
+  "*String which is displayed as heading of MEMO."
+  :group 'mhc
+  :type 'string)
+
 ;;; Internal Variable:
 
 (defconst mhc-summary-major-mode-alist
@@ -245,11 +322,16 @@ third one is replaced with day of month."
 
 (defvar mhc-todo/line-inserter nil)
 
+(defvar mhc-overdue-todo/line-inserter nil)
+
+(defvar mhc-memo/line-inserter nil)
+
 (defvar mhc-summary-line-format-alist
   '((?Y (mhc-summary/line-year-string)
 	'face mhc-tmp-day-face)
     (?/ (if mhc-tmp-first "/" " ")
 	'face mhc-tmp-day-face)
+    (?S " " 'face mhc-tmp-day-face)
     (?M (mhc-summary/line-month-string)
 	'face mhc-tmp-day-face)
     (?D (mhc-summary/line-day-string)
@@ -289,7 +371,19 @@ third one is replaced with day of month."
 	  (mhc-face-category-to-face 
 	   (car (mhc-schedule-categories mhc-tmp-schedule)))))
     (?l (mhc-summary/line-location-string)
-	'face 'mhc-summary-face-location))
+	'face 'mhc-summary-face-location)
+    (?\( (if mhc-tmp-first "(" " ")
+	 'face mhc-tmp-day-face)
+    (?\) (if mhc-tmp-first ")" " ")
+	 'face mhc-tmp-day-face)
+    (?ǯ (if mhc-tmp-first "ǯ" (make-string 2 ? ))
+	 'face mhc-tmp-day-face)
+    (? (if mhc-tmp-first "" (make-string 2 ? ))
+	 'face mhc-tmp-day-face)
+    (? (if mhc-tmp-first "" (make-string 2 ? ))
+	 'face mhc-tmp-day-face)
+    (? (mhc-summary/line-day-of-week-ja-string)
+	 'face mhc-tmp-day-face))
   "An alist of format specifications that can appear in summary lines.
 Each element is a list of following:
 \(SPEC STRING-EXP PROP-TYPE PROP-VALUE\)
@@ -301,7 +395,8 @@ PROP-VALUE is the property value correspond to PROP-TYPE.
 ")
 
 (defvar mhc-todo-line-format-alist
-  '((?i (not mhc-tmp-private) 'icon
+  '((?T "TODO" 'face 'mhc-category-face-todo)
+    (?i (not mhc-tmp-private) 'icon
 	(delete "todo"
 		(delete "done"
 			(copy-sequence
@@ -350,6 +445,38 @@ It indicates a type of the property to put on the inserted string.
 PROP-VALUE is the property value correspond to PROP-TYPE.
 ")
 
+(defvar mhc-memo-line-format-alist
+  '((?i (not mhc-tmp-private) 'icon
+	(if (mhc-schedule-in-category-p mhc-tmp-schedule "done")
+	    (delete "todo"
+		    (copy-sequence (mhc-schedule-categories mhc-tmp-schedule)))
+	  (mhc-schedule-categories mhc-tmp-schedule)))
+    (?s (mhc-summary/line-subject-string)
+	'face
+	(mhc-face-category-to-face 
+	 (car (mhc-schedule-categories mhc-tmp-schedule))))
+    (?l (mhc-summary/line-location-string)
+	'face 'mhc-summary-face-location)
+    (?p (if mhc-tmp-priority
+	    (format "%5s" (format "[%d]" mhc-tmp-priority))
+	  "     ")
+	'face (cond 
+	       ((null mhc-tmp-priority) nil)
+	       ((>= mhc-tmp-priority 80) 'mhc-summary-face-sunday)
+	       ((>= mhc-tmp-priority 50) 'mhc-summary-face-saturday))))
+  "An alist of format specifications that can appear in memo lines.
+Each element is a list of following:
+\(SPEC STRING-EXP PROP-TYPE PROP-VALUE\)
+SPEC is a character for format specification.
+STRING is an expression to get string to insert.
+PROP-TYPE is an expression to get one of the two symbols `face' or `icon'.
+It indicates a type of the property to put on the inserted string.
+PROP-VALUE is the property value correspond to PROP-TYPE.
+")
+
+(defvar mhc-summary/cw-separator nil)
+(defvar mhc-summary/cw-week nil)
+
 ;;; MUA Backend Functions:
 
 (defun mhc-summary-mailer-type ()
@@ -444,18 +571,47 @@ If optional argument FOR-DRAFT is non-nil, Hilight message as draft message."
 
 
 ;;; Codes:
-(defun mhc-summary/insert-separator (&optional wide)
-  (let (hr)
-    (if wide
-	(progn
-	  (setq hr (make-string
-		    (- (window-width) 2) mhc-summary-month-separator))
-	  (mhc-face-put hr 'mhc-summary-face-month-separator))
-      (setq hr (make-string
-		(- (window-width) 24) mhc-summary-separator))
-      (mhc-face-put hr 'mhc-summary-face-separator))
-    (insert hr "\n")))
+(defsubst mhc-summary/make-string (count character)
+  (make-string (max 4 count) character))	;; xxxx 4 ?
 
+(defun mhc-summary/insert-separator (&optional wide str fixwidth)
+  (let ((width (mhc-misc-get-width))
+	hr)
+    (if wide
+	(if (stringp str)
+	    (let ((hr1 (make-string 4 mhc-summary-month-separator))	;; xxxx 4 ?
+		  hr2)
+	      (mhc-face-put hr1 'mhc-summary-face-month-separator)
+	      (mhc-face-put str 'mhc-summary-face-cw)
+	      (setq hr2 (mhc-summary/make-string (- width
+						    (if (numberp mhc-use-month-separator)
+							mhc-calendar-width 2)
+						    (length hr1) (length str))
+						 mhc-summary-month-separator))
+	      (mhc-face-put hr2 'mhc-summary-face-separator)
+	      (setq hr (concat hr1 str hr2)))
+	  (setq hr (mhc-summary/make-string
+		    (if (numberp mhc-use-month-separator)
+			mhc-use-month-separator
+		      (- width 2))
+		    mhc-summary-month-separator))
+	  (mhc-face-put hr 'mhc-summary-face-month-separator))
+      (if (stringp str)
+	  (let ((hr1 (make-string 4 mhc-summary-separator))	;; xxxx 4 ?
+		hr2)
+	    (mhc-face-put hr1 'mhc-summary-face-separator)
+	    (mhc-face-put str 'mhc-summary-face-cw)
+	    (setq hr2 (mhc-summary/make-string (- width mhc-calendar-width
+						  (length hr1) (length str))
+					       mhc-summary-separator))
+	    (mhc-face-put hr2 'mhc-summary-face-separator)
+	    (setq hr (concat hr1 str hr2)))
+	(if fixwidth
+	    (setq hr (mhc-summary/make-string fixwidth mhc-summary-separator))
+	  (setq hr (mhc-summary/make-string (- width mhc-calendar-width)
+					    mhc-summary-separator)))
+	(mhc-face-put hr 'mhc-summary-face-separator)))
+    (insert hr "\n")))
 
 (defvar mhc-summary/today nil)
 
@@ -493,7 +649,7 @@ If optional argument FOR-DRAFT is non-nil, Hilight message as draft message."
 		   (car schedules)
 		   (and secret
 			(mhc-schedule-in-category-p
-			 (car schedules) "private"))
+			 (car schedules) mhc-category-as-private))
 		   'mhc-summary-line-insert
 		   mailer)
 		  (setq mhc-tmp-first nil)))
@@ -509,19 +665,56 @@ If optional argument FOR-DRAFT is non-nil, Hilight message as draft message."
 
 (defun mhc-summary-make-contents
   (from to mailer &optional category-predicate secret)
-  (let ((dayinfo-list (mhc-db-scan from to)))
+  (let ((dayinfo-list (mhc-db-scan from to))
+	todo-list overdue deadline mhc-tmp-day)
     (setq mhc-summary/today (mhc-date-now))
     (while dayinfo-list
       (mhc-summary/insert-dayinfo
        (car dayinfo-list) mailer
        (or category-predicate mhc-default-category-predicate-sexp)
        secret)
-      (and (eq (mhc-day-day-of-week (car dayinfo-list)) mhc-use-week-separator)
+      (when (and mhc-insert-overdue-todo
+		 (mhc-date= (mhc-day-date (car dayinfo-list)) mhc-summary/today))
+	(setq todo-list (mhc-db-scan-todo mhc-summary/today))
+	(while todo-list
+	  (setq deadline (mhc-schedule-todo-deadline (car todo-list)))
+	  (when (and deadline
+		     (if mhc-summary-display-todo
+			 (> (mhc-date- mhc-summary/today deadline) 0)
+		       (>= (mhc-date- mhc-summary/today deadline) 0))
+		     (not (mhc-schedule-in-category-p (car todo-list) "done")))
+	    (setq overdue (cons (car todo-list) overdue)))
+	  (setq todo-list (cdr todo-list)))
+	(setq mhc-tmp-day mhc-summary/today)
+	(setq overdue (nreverse overdue))
+	(while overdue
+	  (mhc-summary-insert-contents
+	   (car overdue)
+	   (and secret
+		(mhc-schedule-in-category-p (car overdue)
+					    mhc-category-as-private))
+	   'mhc-overdue-todo-line-insert
+	   mailer)
+	  (setq overdue (cdr overdue))))
+      (and mhc-use-week-separator
+	   (eq (mhc-day-day-of-week (car dayinfo-list))
+	       (mhc-end-day-of-week))
 	   (> (length dayinfo-list) 1)
-	   (mhc-summary/insert-separator))
+	   (mhc-summary/insert-separator
+	    nil
+	    (when mhc-summary/cw-separator
+	      (format " CW %d " (mhc-date-cw
+				 (mhc-date++ (mhc-day-date (car dayinfo-list))))))))
       (setq dayinfo-list (cdr dayinfo-list)))))
 
 
+(defun mhc-summary-make-todo-memo (today mailer category-predicate secret)
+  (when mhc-insert-todo-list
+    (mhc-summary-make-todo-list today mailer category-predicate secret))
+  (when mhc-insert-memo-list
+    (mhc-summary-make-memo-list today mailer category-predicate secret)))
+
+
 (defun mhc-summary-make-todo-list
   (day mailer &optional category-predicate secret)
   (let ((schedules (mhc-db-scan-todo day))
@@ -539,12 +732,31 @@ If optional argument FOR-DRAFT is non-nil, Hilight message as draft message."
 		(mhc-summary-insert-contents
 		 (car schedules)
 		 (and secret
-		      (mhc-schedule-in-category-p (car schedules) "private"))
+		      (mhc-schedule-in-category-p (car schedules)
+						  mhc-category-as-private))
 		 'mhc-todo-line-insert
 		 mailer))
 	    (setq schedules (cdr schedules)))))))
 
 
+(defun mhc-summary-make-memo-list
+  (day mailer &optional category-predicate secret)
+  (let ((schedules (mhc-db-scan-memo day))
+	(mhc-tmp-day day))
+    (when schedules
+      (insert (format "%s\n" mhc-memo-string-heading))
+      (while schedules
+	(when (funcall category-predicate (car schedules))
+	  (mhc-summary-insert-contents
+	   (car schedules)
+	   (and secret
+		(mhc-schedule-in-category-p (car schedules)
+					    mhc-category-as-private))
+	   'mhc-memo-line-insert
+	   mailer))
+	(setq schedules (cdr schedules))))))
+
+
 (defun mhc-summary/line-year-string ()
   (if mhc-tmp-first
       (format "%4d" (mhc-day-year mhc-tmp-dayinfo))
@@ -565,9 +777,21 @@ If optional argument FOR-DRAFT is non-nil, Hilight message as draft message."
 
 (defun mhc-summary/line-day-of-week-string ()
   (if mhc-tmp-first
-      (format "%s" (mhc-day-day-of-week-as-string mhc-tmp-dayinfo))
+      (let ((week (mhc-day-day-of-week mhc-tmp-dayinfo)))
+	(if (and mhc-summary/cw-week (= week 1) )
+	    (format "%3s"
+		    (format "w%d" (mhc-date-cw (mhc-day-date mhc-tmp-dayinfo))))
+	  (aref ["Sun" "Mon" "Tue" "Wed" "Thu" "Fri" "Sat"] week)))
     (make-string 3 ? )))
 
+(defun mhc-summary/line-day-of-week-ja-string ()
+  (if mhc-tmp-first
+      (let ((week (mhc-day-day-of-week mhc-tmp-dayinfo)))
+	(if (and mhc-summary/cw-week(= week 1) )
+	    (format "%2d" (mhc-date-cw (mhc-day-date mhc-tmp-dayinfo)))
+	  (aref ["" "" "" "" "" "" ""] week)))
+    (make-string 2 ? )))
+
 
 (defun mhc-summary/line-subject-string ()
   (if mhc-tmp-private
@@ -663,6 +887,11 @@ If optional argument FOR-DRAFT is non-nil, Hilight message as draft message."
   (if (and (interactive-p)
 	   (mhc-use-icon-p))
       (call-interactively 'mhc-icon-setup))
+  (setq mhc-summary/cw-separator (and mhc-summary-use-cw
+				      mhc-use-week-separator
+				      (eq mhc-start-day-of-week 1)))
+  (setq mhc-summary/cw-week (and mhc-summary-use-cw
+				 (not mhc-summary/cw-separator)))
   (mhc-line-inserter-setup
    mhc-summary/line-inserter
    mhc-summary-line-format
@@ -670,8 +899,16 @@ If optional argument FOR-DRAFT is non-nil, Hilight message as draft message."
   (mhc-line-inserter-setup
    mhc-todo/line-inserter
    mhc-todo-line-format
-   mhc-todo-line-format-alist))
-  
+   mhc-todo-line-format-alist)
+  (mhc-line-inserter-setup
+   mhc-overdue-todo/line-inserter
+   mhc-overdue-todo-line-format
+   mhc-todo-line-format-alist)
+  (mhc-line-inserter-setup
+   mhc-memo/line-inserter
+   mhc-memo-line-format
+   mhc-memo-line-format-alist))
+
 
 (defun mhc-summary-line-insert ()
   "Insert summary line."
@@ -698,6 +935,17 @@ If optional argument FOR-DRAFT is non-nil, Hilight message as draft message."
 	(mhc-tmp-priority (mhc-schedule-priority mhc-tmp-schedule)))
     (funcall mhc-todo/line-inserter)))
 
+(defun mhc-overdue-todo-line-insert ()
+  "Insert overdue todo line."
+  (let ((mhc-tmp-deadline (mhc-schedule-todo-deadline mhc-tmp-schedule))
+	(mhc-tmp-priority (mhc-schedule-priority mhc-tmp-schedule)))
+    (funcall mhc-overdue-todo/line-inserter)))
+
+(defun mhc-memo-line-insert ()
+  "Insert memo line."
+  (let ((mhc-tmp-priority (mhc-schedule-priority mhc-tmp-schedule)))
+    (funcall mhc-memo/line-inserter)))
+
 
 (provide 'mhc-summary)
 
