@@ -3,7 +3,7 @@
 ## Author:  Yoshinari Nomura <nom@quickhack.net>
 ##
 ## Created: 1999/09/01
-## Revised: $Date: 2001/02/06 13:36:01 $
+## Revised: 2009-01-07 00:15:05
 ##
 
 require 'mhc-date'
@@ -128,6 +128,10 @@ class PilotDB
     PiLib .dlp_ResetSyncFlags(@sd, @db)
     return self
   end
+
+  def get_app_info()
+    return PiLib .dlp_ReadAppBlock(@sd, @db)
+  end
 end
 
 ################################################################
@@ -185,7 +189,7 @@ class PilotRecord
     attr_str << 'Busy'     if (@attr & 0x20 != 0)
     attr_str << 'Secret'   if (@attr & 0x10 != 0)
     attr_str << 'Archived' if (@attr & 0x08 != 0)
-    return attr_str .join (' ')
+    return attr_str .join(' ')
   end
 
   attr :category
@@ -513,7 +517,7 @@ class PilotApptRecord < PilotRecord
     set_frequency(freq)    ## freq     ηå⤹
 
     ## beg դ ord, week Ƥ뤫Υåɬ
-    msg = "Type/ange error: (0< ord <4, 0< wek <6) required. (#{ord}, #{wek})"
+    msg = "Type/Range error: (0< ord <4, 0< wek <6) required. (#{ord}, #{wek})"
     raise msg if !(ord .is_a?(Integer) && 0 <= ord && ord <= 4)
     raise msg if !(wek .is_a?(Integer) && 0 <= wek && wek <= 6)
     raise msg if !((ord == 4 && beg .o_last?) || (ord < 4  && beg .o == ord))
@@ -579,13 +583,20 @@ class PilotApptRecord < PilotRecord
     xsc["Record-Id"]  = create_message_id(@id .to_s) # xxx
     xsc["Pilot-Attr"] = attribute_string
     xsc["Pilot-Id"]   = @id
-    xsc["Subject"]    = Kconv::tojis(@description)
+    xsc["Subject"]    = Kconv::tojis(Kconv::toeuc(@description) .sub(/\[[^\]]*\]\s*$/, ''))
+    xsc["Location"]   = Kconv::tojis($1) if Kconv::toeuc(@description) =~ /\[([^\]]+)\]\s*$/
     xsc["Note"]       = Kconv::tojis(@note)
     xsc["Category"]   = @category  if category?
     xsc["Alarm"]      = alarm
     xsc["Day"]        = @exception .collect{|t| '!' + t .to_xscday} .join(' ')
     xsc["Day"]       += ' ' + @beg .to_xscday if !repeat?
-    xsc["Time"]  = ' ' + @beg .to_xsctime + '-' + @fin .to_xsctime if !event?
+    if !event?
+      if @beg .to_xsctime == @fin .to_xsctime
+        xsc["Time"]  = ' ' + @beg .to_xsctime
+      else
+        xsc["Time"]  = ' ' + @beg .to_xsctime + '-' + @fin .to_xsctime
+      end
+    end
 
     if repeat?
       if @repeatFrequency > 1
@@ -657,8 +668,8 @@ class PilotApptRecord < PilotRecord
       part1_is_header = false
     end
 
-    part1 .split("\n") .each{|line|
-      if !(string =~ /^[ \t]+/ or string =~ /^[A-Za-z0-9_-]+:/)
+    part1 .to_s .split("\n") .each{|line|
+      if !(line =~ /^[ \t]+/ or line =~ /^[A-Za-z0-9_-]+:/)
 	part1_is_header = false
       end
     }
@@ -778,7 +789,7 @@ class PilotAddressDB < PilotDB
     ## @phoneLabels[0..7] = 
     ##      [, , Fax, ¾, E-mail, ɽ, ݥ٥, ]
 
-    @recClass = PilotAddRec
+    @recClass = PilotAddRecord
   end
 end
 
